

CREATE OR REPLACE FUNCTION spJnl_LgrVoid (
	_JnlNo		VARCHAR(15)
) RETURNS SETOF SPStatus AS 
$BODY$
DECLARE
	_retval		SPStatus%ROWTYPE;
	_spret		SPStatus%ROWTYPE;
	cAcc		REFCURSOR;
	_AccNo		VARCHAR(10);
	_tgl		DATE;
	_amount		NUMERIC(19,4);
	_DK		VARCHAR(1);
	_DefDK		VARCHAR(1);
	_isAutoGen	BOOLEAN;
	_LockDate	DATE;
	_BranchID	VARCHAR(5);
	_DeptID		VARCHAR(5);
	_PrjID		VARCHAR(5);
	_PerStart	DATE;
BEGIN
	_retval.ErrNo:=50001;
	_retval.ErrMsg:='General Error (spJnl_Void)';

	SELECT INTO 
		_tgl, _isAutoGen
		jnldate, isAutoGen
	FROM jnl WHERE JnlNo=_JnlNo;

	SELECT INTO _PerStart perstart from per where active=true;
	IF(_PerStart>_tgl) THEN
		_retval.ErrNo:=50002;
		_retval.ErrMsg='Transaksi Melebihi Batas Periode Akuntansi ! (spJnl_Void)';
		RAISE EXCEPTION '%', _retval.ErrMsg;
	END IF;

	SELECT INTO _LockDate lockdate FROM TLock WHERE TransType='JNL';
	IF(_LockDate is not null AND _LockDate>_tgl) THEN
		_retval.ErrNo:=50002;
		_retval.ErrMsg='Jurnal Melebihi Batas Tanggal Kunci Transaksi ! (spJnl_Void)';
		RAISE EXCEPTION '%', _retval.ErrMsg;
	END IF;

	OPEN cAcc FOR SELECT d.accno,d.dk,d.jnldamt,j.branchid,d.deptid,d.prjid FROM jnl j, jnld d WHERE j.jnlno=d.jnlno
	AND d.JnlNo=_JnlNo;
	LOOP
		FETCH cAcc INTO _AccNo,_DK,_amount,_BranchID,_DeptID,_PrjID;
		EXIT WHEN NOT FOUND;
		SELECT INTO _DefDK accdk FROM acc WHERE AccNo=_AccNo;
		IF _DefDK=_DK THEN	
			_amount:=_amount*-1;
		END IF;			
		
		SELECT INTO _spret * FROM spAccBal_Update(_AccNo,_tgl,_amount,_BranchID,_DeptID,_PrjID);
		IF _spret.ErrNo<>0 THEN
			_retval.ErrNo:=_spret.ErrNo;
			_retval.ErrMsg:=_spret.ErrMsg;
			RAISE EXCEPTION '%', _retval.ErrMsg;
		END IF;
		
	END LOOP;

	/** Void nested trans: BPA, CashA */

	IF (_isAutoGen=false) THEN
		SELECT INTO _spret * FROM spBPA_Void_ByRef('JNL', _JnlNo);
		IF _spret.ErrNo<>0 THEN
			_retval.ErrNo:=_spret.ErrNo;
			_retval.ErrMsg:=_spret.ErrMsg;
			RAISE EXCEPTION '%', _retval.ErrMsg;
		END IF;

		SELECT INTO _spret * FROM spCashA_Void_ByRef('JNL', _JnlNo, NULL, NULL);
		IF _spret.ErrNo<>0 THEN
			_retval.ErrNo:=_spret.ErrNo;
			_retval.ErrMsg:=_spret.ErrMsg;
			RAISE EXCEPTION '%', _retval.ErrMsg;
		END IF;		
	END IF;
	/***/

	_retval:=(50010, 'Error Delete Lgr (spJnl_Void)');
	DELETE FROM Lgr WHERE jnlno=_JnlNo;

	_retval:=(0, 'OK');
	RETURN NEXT _retval;
END;
$BODY$ 
LANGUAGE 'plpgsql' VOLATILE;




CREATE OR REPLACE FUNCTION spFixJurnalRevaluasiPayRcv()
RETURNS SETOF SPStatus AS
$BODY$
DECLARE
	_retval		SPStatus%ROWTYPE;
	_spret		SPStatus%ROWTYPE;

	cData		REFCURSOR;
	cPaid		REFCURSOR;

	_JnlNo		jnl.jnlno%TYPE;
	_RefType	reftype.reftype%TYPE;
	_RefNo		VARCHAR(15);
	_AccNo		acc.accno%TYPE;
	_BPId		bp.bpid%TYPE;
	_JnlAmt		NUMERIC(19,4);

	_PaidRefType	reftype.reftype%TYPE;
	_PaidRefNo	VARCHAR(15);
	_PayStatus	VARCHAR(1);

	_SelisihKurs	NUMERIC(19,4);
	_Selisih	NUMERIC(19,4);
	_CrcIdDefault	crc.crcid%TYPE;
BEGIN
	_retval:=(0, 'OK');

	RAISE NOTICE E'\nProses akan memakan waktu tergantung berapa banyak data yang salah. Silahkan tunggu hingga proses selesai.\nJangan menutup window untuk menghindari hal-hal yang tidak diinginkan.';

	SELECT INTO _CrcIdDefault crcid 
	FROM crc 
	WHERE isdefault=TRUE;

	OPEN cData FOR
		SELECT jnlno, refno, reftype, jnlamt 
		FROM jnl
		WHERE jnlnote LIKE 'Revaluasi Kurs Pembayaran%' AND reftype='PAY';
	LOOP
		FETCH cData INTO _JnlNo, _RefNo, _RefType, _JnlAmt;
		EXIT WHEN NOT FOUND;

		RAISE NOTICE 'Processing Jurnal#% atas payment %', _JnlNo, _RefNo;
		_SelisihKurs:=0;

		OPEN cPaid FOR
			SELECT reftype, refno 
			FROM ppaid 
			WHERE payno=_RefNo AND crcid<>_CrcIdDefault AND reftype IN ('PURC', 'PRET');
		LOOP
			FETCH cPaid INTO _PaidRefType, _PaidRefNo;
			EXIT WHEN NOT FOUND;

			IF(_PaidRefType='PURC') THEN
				SELECT INTO _PayStatus paystatus 
				FROM purc WHERE purcno=_PaidRefNo;

				IF(_PayStatus='F') THEN
					SELECT INTO _Selisih * FROM fCheckSaldo(_PaidRefType, _PaidRefNo);
				END IF;
			ELSIF(_PaidRefType='PRET') THEN
				SELECT INTO _Selisih * FROM fCheckSaldo(_PaidRefType, _PaidRefNo);
			END IF;

			_SelisihKurs:=_SelisihKurs+_Selisih;
		END LOOP;
		CLOSE cPaid;

		_SelisihKurs:=ABS(_SelisihKurs);
		IF (_SelisihKurs!=_JnlAmt) THEN
			/** Void lgr */
			SELECT INTO _spret * FROM spJnl_LgrVoid(_JnlNo);
			IF (_spret.ErrNo<>0) THEN
				_retval.ErrNo:=_spret.ErrNo;
				_retval.ErrMsg:=_spret.ErrMsg;
				RAISE EXCEPTION '%', _retval.ErrMsg;
			END IF;

			UPDATE jnl SET jnlamt=_SelisihKurs WHERE jnlno=_JnlNo;
			UPDATE jnld SET debit=_SelisihKurs, jnldamt=_SelisihKurs, jnldamtforex=_SelisihKurs WHERE jnlno=_JnlNo AND debit!=0 AND credit=0;
			UPDATE jnld SET credit=_SelisihKurs, jnldamt=_SelisihKurs, jnldamtforex=_SelisihKurs WHERE jnlno=_JnlNo AND credit!=0 AND debit=0;

			/** execute ulang spjnl_new */
			SELECT INTO _spret * FROM spJnl_New(_JnlNo);
			IF (_spret.ErrNo<>0) THEN
				_retval.ErrNo:=_spret.ErrNo;
				_retval.ErrMsg:=_spret.ErrMsg;
				RAISE EXCEPTION '%', _retval.ErrMsg;
			END IF;
		END IF;
	END LOOP;
	CLOSE cData;

	OPEN cData FOR
		SELECT jnlno, refno, reftype, jnlamt 
		FROM jnl
		WHERE jnlnote LIKE 'Revaluasi Kurs Penerimaan%' AND reftype='RCV';
	LOOP
		FETCH cData INTO _JnlNo, _RefNo, _RefType, _JnlAmt;
		EXIT WHEN NOT FOUND;

		RAISE NOTICE 'Processing Jurnal#% atas penerimaan %', _JnlNo, _RefNo;
		_SelisihKurs:=0;

		OPEN cPaid FOR
			SELECT reftype, refno 
			FROM spaid 
			WHERE rcvno=_RefNo AND crcid<>_CrcIdDefault AND reftype IN ('SALE', 'SRET');
		LOOP
			FETCH cPaid INTO _PaidRefType, _PaidRefNo;
			EXIT WHEN NOT FOUND;

			IF(_PaidRefType='SALE') THEN
				SELECT INTO _PayStatus paystatus 
				FROM sale WHERE saleno=_PaidRefNo;

				IF(_PayStatus='F') THEN
					SELECT INTO _Selisih * FROM fCheckSaldo(_PaidRefType, _PaidRefNo);
				END IF;
			ELSIF(_PaidRefType='SRET') THEN
				SELECT INTO _Selisih * FROM fCheckSaldo(_PaidRefType, _PaidRefNo);
			END IF;

			_SelisihKurs:=_SelisihKurs+_Selisih;
		END LOOP;
		CLOSE cPaid;

		_SelisihKurs:=ABS(_SelisihKurs);
		IF (_SelisihKurs!=_JnlAmt) THEN
			/** Void lgr */
			SELECT INTO _spret * FROM spJnl_LgrVoid(_JnlNo);
			IF (_spret.ErrNo<>0) THEN
				_retval.ErrNo:=_spret.ErrNo;
				_retval.ErrMsg:=_spret.ErrMsg;
				RAISE EXCEPTION '%', _retval.ErrMsg;
			END IF;

			UPDATE jnl SET jnlamt=_SelisihKurs WHERE jnlno=_JnlNo;
			UPDATE jnld SET debit=_SelisihKurs, jnldamt=_SelisihKurs, jnldamtforex=_SelisihKurs WHERE jnlno=_JnlNo AND debit!=0 AND credit=0;
			UPDATE jnld SET credit=_SelisihKurs, jnldamt=_SelisihKurs, jnldamtforex=_SelisihKurs WHERE jnlno=_JnlNo AND credit!=0 AND debit=0;

			/** execute ulang spjnl_new */
			SELECT INTO _spret * FROM spJnl_New(_JnlNo);
			IF (_spret.ErrNo<>0) THEN
				_retval.ErrNo:=_spret.ErrNo;
				_retval.ErrMsg:=_spret.ErrMsg;
				RAISE EXCEPTION '%', _retval.ErrMsg;
			END IF;
		END IF;
	END LOOP;
	CLOSE cData;

	RETURN NEXT _retval;
END;
$BODY$
LANGUAGE 'plpgsql' VOLATILE;




CREATE OR REPLACE FUNCTION spFixTanggalJurnalRekonsiliasi() 
RETURNS SETOF SPStatus AS
$BODY$
DECLARE
	_retval		SPStatus%ROWTYPE;
	_spret		SPStatus%ROWTYPE;

	cData		REFCURSOR;

	_JnlNo		jnl.jnlno%TYPE;
	_JnlDate	jnl.jnldate%TYPE;
	_RefNo		VARCHAR(15);
	_RefType	reftype.reftype%TYPE;

BEGIN
	_retval:=(0, 'OK');
	RAISE NOTICE E'\nProses akan memakan waktu tergantung berapa banyak data yang salah. Silahkan tunggu hingga proses selesai.\nJangan menutup window untuk menghindari hal-hal yang tidak diinginkan.';
	OPEN cData FOR
		SELECT jnlno, reftype, refno 
		FROM jnl 
		WHERE jnlnote LIKE 'Rekonsiliasi Bank%' AND reftype!='CBG';
	LOOP
		FETCH cData INTO _JnlNo, _RefType, _RefNo;
		EXIT WHEN NOT FOUND;

		RAISE NOTICE 'Processing Jurnal#% atas % : %', _JnlNo, _RefType, _RefNo;

		SELECT INTO _spret * FROM spJnl_LgrVoid(_JnlNo);
		IF (_spret.ErrNo<>0) THEN
			_retval.ErrNo:=_spret.ErrNo;
			_retval.ErrMsg:=_spret.ErrMsg;
			RAISE EXCEPTION '%', _retval.ErrMsg;
		END IF;

		SELECT INTO _JnlDate cashadate 
		FROM casha WHERE reftype=_RefType AND refno=_RefNo LIMIT 1;
		IF FOUND THEN
			UPDATE jnl SET jnldate=_JnlDate WHERE jnlno=_JnlNo;

			SELECT INTO _spret * FROM spJnl_New(_JnlNo);
			IF (_spret.ErrNo<>0) THEN
				_retval.ErrNo:=_spret.ErrNo;
				_retval.ErrMsg:=_spret.ErrMsg;
				RAISE EXCEPTION '%', _retval.ErrMsg;
			END IF;
		END IF;
	END LOOP;
	CLOSE cData;

	RETURN NEXT _retval;
END;
$BODY$
LANGUAGE 'plpgsql' VOLATILE;


SELECT * FROM spFixJurnalRevaluasiPayRcv();


SELECT * FROM spFixTanggalJurnalRekonsiliasi();


